!
! Copyright (C) 2014-2015 Orbital-free DFT group at University of Florida
!
! This program is free software; you can redistribute it and/or modify 
! it under the terms of the GNU General Public License as published by 
! the Free Software Foundation; either version 2 of the License, or 
! any later version. See the file LICENSE in the root directory of the 
! present distribution, or http://www.gnu.org/copyleft/gpl.txt ,
! or contact developers via e-mail: vkarasev@qtp.ufl.edu , or paper mail:
!
! Quantum Theory Project
! University of Florida
! P.O. Box 118435
! Gainesville, FL 32611
!
! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.
!-----------------------------------------------------------------------
! Test of the KSDT XC functional implementation in LibXC library
! Data from Table S1 of Supplemental for PRL 112, 076403 (2014) are reproduced.
! Functional derivatives w.r.t. spin-up and spin-down densities are calculated
! numerically and compared to values returned by LibXC.
! Please be aware that the centered 5-point finite-difference formula will fail
! for zeta equal or very close to 1.
! KSDT and Perdew-Zunger (PZ) XC values are compared at T=0K.
! Please refer to the PRL 112, 076403 (2014) for defitinions of reduced
! temperature at zeta=0, and zeta=1 and for convention of using t(zeta=0) in Eq.
! (17), though the second half of Table S1 refers to values corresponding to
! t(zeta=1).
!
! Authors: Lazaro Calderin and Valentin Karasiev
!-----------------------------------------------------------------------
! 
program lxcksdttest
   use xc_f90_types_m
   use xc_f90_lib_m

   implicit none

   real(8), parameter :: pi = 4.0d0*atan(1.0d0)
   real(8), parameter :: onethird = 1.0d0/3.0d0
   real(8), parameter :: twothird = 2.0d0/3.0d0;


   real(8) :: rs(7) =(/1.0d0, 2.0d0, 4.0d0, 6.0d0, 8.0d0, 10.0d0, 40.0d0/)
   real(8) :: tred(8)=(/0.0625d0,0.125d0,0.25d0,0.5d0,1.d0,2.d0,4.d0,8.d0/)
   real(8) :: T,TFermi
   real(8) :: rho(7),exc(7),fxc(7),vxc(7)
   real(8) :: fxc0(7,8),fxc1(7,8) !fxc(rs,t) for unpolarized and polarized case
   real(8) :: zeta,phi(7)
   real(8) :: ex(7),vx(7)
   real(8) :: ec(7),vc(7)
   real(8) :: rhop(2,7),vxp(2,7),vcp(2,7)
   real(8) :: vxcp(2,7)
   
   real(8) :: drho,vxpnum,vcnum
   real(8) :: extmp(2,5),ectmp(2,5),exctmp(2,5)
   real(8) :: fxctmp(2,5),rhotmp
   real(8) :: exctmp0(5)

   integer :: ii, SPIN,jj,kk
   integer :: vmajor, vminor, vmicro

  TYPE(xc_f90_pointer_t) :: x_func, c_func, ksdt_func
  TYPE(xc_f90_pointer_t) :: x_info, c_info, ksdt_info

   call xc_f90_version(vmajor, vminor, vmicro)
   write(*,'("# Libxc version: ",I1,".",I1,".",I1)') vmajor, vminor, vmicro

!-----------------------------------------------------------------------
! KSDT XC, reproduce data of Table S1, PRL 112, 076403 (2014) SuppMatt
! spin-unpolarized case, , 
!-----------------------------------------------------------------------
   SPIN=XC_UNPOLARIZED
   call xc_f90_func_init(ksdt_func, ksdt_info, XC_LDA_XC_KSDT, SPIN)

   write(*,*)
   write(*,*) '# a.) KSDT XC:'
   write(*,*) '# spin-unpolarized case, first half of Table S1'
   write(*,*) '#         rs            t            fxc           vxc'
   do jj=1,8
   do ii=1,7
      rho(ii) = 3.0d0/( 4.0d0 * pi * rs(ii) * rs(ii) * rs(ii) )
      TFermi=(3.d0*pi**2*rho(ii))**(2.d0/3.d0)/2.d0 !Fermi temperature zeta=0
      T=tred(jj)*TFermi
      rhop(1,ii) = rho(ii)
      rhop(2,ii) = 0.d0
      call xc_f90_lda_xc_ksdt_set_par(ksdt_func, T);
      call xc_f90_lda_exc_vxc(ksdt_func,1,rho(ii),fxc(ii),vxc(ii))
      write(*,'(5f14.4)') rs(ii), tred(jj), fxc(ii), vxc(ii)
      fxc0(ii,jj) = fxc(ii)
   enddo
   write(*,*)'#'
   enddo

!-----------------------------------------------------------------------
! KSDT XC, reproduce data of Table S1, PRL 112, 076403 (2014) SuppMatt
! fully-spin-polarized case, , 
!-----------------------------------------------------------------------
   SPIN=XC_POLARIZED
   call xc_f90_func_init(ksdt_func, ksdt_info, XC_LDA_XC_KSDT, SPIN)

   write(*,*)
   write(*,*) '# b.) KSDT XC:'
   write(*,*) '# fully-spin-polarized case, second half of Table S1'
   write(*,*) '#         rs            t            fxc           vxc'
   do jj=1,8
   do ii=1,7
      rho(ii) = 3.0d0/( 4.0d0 * pi * rs(ii) * rs(ii) * rs(ii) )
      TFermi=(6.d0*pi**2*rho(ii))**(2.d0/3.d0)/2.d0 !Fermi temperature zeta=1
      T=tred(jj)*TFermi
      call xc_f90_lda_xc_ksdt_set_par(ksdt_func, T);
      call xc_f90_lda_exc_vxc(ksdt_func,1,rhop(1,ii),fxc(ii),vxcp(1,ii))
      write(*,'(5f14.4)') rs(ii), tred(jj), fxc(ii), vxcp(1,ii)

      TFermi=(3.d0*pi**2*rho(ii))**(2.d0/3.d0)/2.d0 !Fermi temperature zeta=0,
      !use TFermi(zeta=0) definition to reproduce value of phi
      T=tred(jj)*TFermi
      call xc_f90_lda_xc_ksdt_set_par(ksdt_func, T);
      call xc_f90_lda_exc_vxc(ksdt_func,1,rhop(1,ii),fxc(ii),vxcp(1,ii))
      fxc1(ii,jj) = fxc(ii)
   enddo
   write(*,*)'#'
   enddo
!-----------------------------------------------------------------------
! KSDT XC, reproduce data of Table S1, PRL 112, 076403 (2014) SuppMatt
! zeta=0.34, values of phi
!-----------------------------------------------------------------------
   SPIN=XC_POLARIZED
   call xc_f90_func_init(ksdt_func, ksdt_info, XC_LDA_XC_KSDT, SPIN)

   zeta = 0.34d0 !intermediate spin-polarization
   write(*,*)
   write(*,*) '# c.) KSDT XC: intermediate polarization and last column of Table S1'
   write(*,*) '# Please do not be confused about t-dependence of fxc(zeta=1) in Eq.(17),'
   write(*,*) '# fxc(rs,zeta,t)=fxc^0(rs,t)+(fxc^1(rs,t/2^2/3)-fxc^0(rs,t))*phi(rs,zeta,t)'
   write(*,*) '# The above test case correspond to fxc^1(rs,t)'
   write(*,*) '#'
   write(*,*) '# analytical derivatives'
   write(*,*) '# zeta=',zeta
   write(*,*) '#         rs            t            fxc           vxc1          vxc2       phi'
   do jj=1,8
   do ii=1,7
      rho(ii) = 3.0d0/( 4.0d0 * pi * rs(ii) * rs(ii) * rs(ii) )
      TFermi=(3.d0*pi**2*rho(ii))**(2.d0/3.d0)/2.d0 !Fermi temperature zeta=0
      T=tred(jj)*TFermi
      rhop(1,ii) = rho(ii)*(1.d0+zeta)/2.d0
      rhop(2,ii) = rho(ii)*(1.d0-zeta)/2.d0
      call xc_f90_lda_xc_ksdt_set_par(ksdt_func, T);
      call xc_f90_lda_exc_vxc(ksdt_func,1,rhop(1,ii),fxc(ii),vxcp(1,ii))
      phi(ii) = (fxc(ii)-fxc0(ii,jj))/(fxc1(ii,jj)-fxc0(ii,jj))
      write(*,'(6f14.4)') rs(ii), tred(jj), fxc(ii), vxcp(1,ii), vxcp(2,ii), phi(ii)
   enddo
   write(*,*)'#'
   enddo
!-----------------------------------------------------------------------
! KSDT XC, intermediate-polarization case, numerical derivatives
!-----------------------------------------------------------------------
   SPIN=XC_POLARIZED
   call xc_f90_func_init(ksdt_func, ksdt_info, XC_LDA_XC_KSDT, SPIN)

   write(*,*)
   write(*,*) '# d.) KSDT XC: intermediate polarization'
   write(*,*) '# numerical derivatives, compare to above table'
   write(*,*) '# zeta=',zeta
   write(*,*) '#         rs            t            fxc           vxc1          vxc2'
   do jj=1,8
   do ii=1,7
      rho(ii) = 3.0d0/( 4.0d0 * pi * rs(ii) * rs(ii) * rs(ii) )
      drho = 0.005d0*rho(ii)
      TFermi=(3.d0*pi**2*rho(ii))**(2.d0/3.d0)/2.d0 !Fermi temperature zeta=0
      T=tred(jj)*TFermi


      do kk=1,5
        rhop(1,ii) = rho(ii)*(1.d0+zeta)/2.d0 + drho*dble(kk-3)
        rhop(2,ii) = rho(ii)*(1.d0-zeta)/2.d0 
        call xc_f90_lda_xc_ksdt_set_par(ksdt_func, T);
        call xc_f90_lda_exc_vxc(ksdt_func,1,rhop(1,ii),fxc(ii),vxcp(1,ii))
        fxctmp(1,kk) = fxc(ii)*(rho(ii) + drho*dble(kk-3))
      enddo

      do kk=1,5
        rhop(1,ii) = rho(ii)*(1.d0+zeta)/2.d0 
        rhop(2,ii) = rho(ii)*(1.d0-zeta)/2.d0 + drho*dble(kk-3)
        call xc_f90_lda_xc_ksdt_set_par(ksdt_func, T);
        call xc_f90_lda_exc_vxc(ksdt_func,1,rhop(1,ii),fxc(ii),vxcp(1,ii))
        fxctmp(2,kk) = fxc(ii)*(rho(ii) + drho*dble(kk-3))
      enddo

      fxc(ii) = fxctmp(1,3)/rho(ii)
      vxcp(1,ii) = fxctmp(1,1)-8.d0*fxctmp(1,2)+8.d0*fxctmp(1,4)-fxctmp(1,5)
      vxcp(1,ii) = vxcp(1,ii)/(12.d0*drho)
      vxcp(2,ii) = fxctmp(2,1)-8.d0*fxctmp(2,2)+8.d0*fxctmp(2,4)-fxctmp(2,5)
      vxcp(2,ii) = vxcp(2,ii)/(12.d0*drho)
      write(*,'(6f14.4)') rs(ii), tred(jj), fxc(ii), vxcp(1,ii), vxcp(2,ii)
   enddo
   write(*,*)'#'
   enddo

!-----------------------------------------------------------------------
! KSDT at T=0, unpolarized case
!-----------------------------------------------------------------------

   T=0.0d0
   SPIN=XC_UNPOLARIZED

   call xc_f90_func_init(ksdt_func, ksdt_info, XC_LDA_XC_KSDT, SPIN)
   call xc_f90_lda_xc_ksdt_set_par(ksdt_func, T)

   do ii=1,7
      rho(ii) = 3.0d0/( 4.0d0 * pi * rs(ii) * rs(ii) * rs(ii) )
      rhop(1,ii) = rho(ii)
      rhop(2,ii) = 0.d0
   enddo
   call xc_f90_lda_exc_vxc(ksdt_func,7,rho(1),fxc(1),vxc(1))

   write(*,*) '# e.) KSDT XC, T=0, unpolarized:'
   write(*,*) '#         rs            T            fxc           vxc'
   do ii = 1, 7
      write(*,'(5f14.4)') rs(ii), T, fxc(ii), vxc(ii)
   end do
!-----------------------------------------------------------------------  
! LDA (Slater+PZ), unpolarized case
!-----------------------------------------------------------------------

   SPIN=XC_UNPOLARIZED
   write(*,*)
   write(*,*) '# f.) Slater(X)+PZ(C), unpolarized:'
   write(*,*) '#         rs           exc           vxc'
  call xc_f90_func_init(x_func, x_info, XC_LDA_X, SPIN)
  call xc_f90_func_init(c_func, c_info, XC_LDA_C_PZ, SPIN)

   call xc_f90_lda_exc_vxc(x_func,7,rho(1),ex(1),vx(1))
   call xc_f90_lda_exc_vxc(c_func,7,rho(1),ec(1),vc(1))

   do ii = 1, 7
      exc(ii)=ex(ii)+ec(ii)
      vxc(ii)=vx(ii)+vc(ii)
      write(*,'(5f14.4)') rs(ii),  exc(ii),  vxc(ii)
   end do

!-----------------------------------------------------------------------
! KSDT at T=0, fully-polarized case
!-----------------------------------------------------------------------
 
    T=0.0d0
    SPIN=XC_POLARIZED
 
    call xc_f90_func_init(ksdt_func, ksdt_info, XC_LDA_XC_KSDT, SPIN)
    call xc_f90_lda_xc_ksdt_set_par(ksdt_func, T)
 
    do ii=1,7
       rho(ii) = 3.0d0/( 4.0d0 * pi * rs(ii) * rs(ii) * rs(ii) )
       rhop(1,ii) = rho(ii)
       rhop(2,ii) = 0.d0
    enddo
    call xc_f90_lda_exc_vxc(ksdt_func,7,rhop(1,1),fxc(1),vxcp(1,1))
 
    write(*,*)
    write(*,*) '# g.) KSDT XC, T=0, fully-polarized:'
    write(*,*) '#         rs            T            fxc           vxc'
    do ii = 1, 7
       write(*,'(5f14.4)') rs(ii), T, fxc(ii), vxcp(1,ii)
    end do
   
!-----------------------------------------------------------------------
! LDA (Slater+PZ), fully-polarized case
!-----------------------------------------------------------------------
 
    SPIN=XC_POLARIZED
    write(*,*)
    write(*,*) '# h.) Slater(X)+PZ(C) fully-polarized:'
    write(*,*) '#         rs           exc           vxc'
    call xc_f90_func_init(x_func, x_info, XC_LDA_X, SPIN)
    call xc_f90_func_init(c_func, c_info, XC_LDA_C_PZ, SPIN)
 
    call xc_f90_lda_exc_vxc(x_func,7,rhop(1,1),ex(1),vxp(1,1))
    call xc_f90_lda_exc_vxc(c_func,7,rhop(1,1),ec(1),vcp(1,1))
 
    do ii = 1, 7
       exc(ii)=ex(ii)+ec(ii)
       vxc(ii)=vxp(1,ii)+vcp(1,ii)
       write(*,'(5f14.4)') rs(ii),  exc(ii),  vxc(ii)
    end do
 
!-----------------------------------------------------------------------
! KSDT at T=0, intermediate polarization
!-----------------------------------------------------------------------
   SPIN=XC_POLARIZED
   call xc_f90_func_init(ksdt_func, ksdt_info, XC_LDA_XC_KSDT, SPIN)

   zeta = 0.34d0 !intermediate spin-polarization
   write(*,*)
   write(*,*) '# i.) KSDT XC, T=0, intermediate polarization:'
   write(*,*) '# analytical derivatives'
   write(*,*) '# zeta=',zeta
   write(*,*) '#         rs            t            fxc           vxc1          vxc2'
   do jj=1,1 !8
   tred(jj) = 0.000001d0
   do ii=1,7
      rho(ii) = 3.0d0/( 4.0d0 * pi * rs(ii) * rs(ii) * rs(ii) )
      TFermi=(3.d0*pi**2*rho(ii))**(2.d0/3.d0)/2.d0 !Fermi temperature zeta=0
      T=tred(jj)*TFermi
      rhop(1,ii) = rho(ii)*(1.d0+zeta)/2.d0
      rhop(2,ii) = rho(ii)*(1.d0-zeta)/2.d0
      call xc_f90_lda_xc_ksdt_set_par(ksdt_func, T);
      call xc_f90_lda_exc_vxc(ksdt_func,1,rhop(1,ii),fxc(ii),vxcp(1,ii))
      write(*,'(6f14.4)') rs(ii), tred(jj), fxc(ii), vxcp(1,ii), vxcp(2,ii)
   enddo
   write(*,*)'#'
   enddo
!
!-----------------------------------------------------------------------
! LDA (Slater+PZ), intermediate-polarization
!-----------------------------------------------------------------------

   SPIN=XC_POLARIZED
   call xc_f90_func_init(x_func, x_info, XC_LDA_X, SPIN)
   call xc_f90_func_init(c_func, c_info, XC_LDA_C_PZ, SPIN)

   write(*,*)
   write(*,*) '# j.) Slater(X)+PZ(C) intermediate polarization:'
   write(*,*) '# analytical derivatives'
   write(*,*) '# zeta=',zeta
   write(*,*) '#         rs           exc           vxc1      vxc2'
   do ii=1,7
      rho(ii) = 3.0d0/( 4.0d0 * pi * rs(ii) * rs(ii) * rs(ii) )
      rhop(1,ii) = rho(ii)*(1.d0+zeta)/2.d0
      rhop(2,ii) = rho(ii)*(1.d0-zeta)/2.d0
      call xc_f90_lda_exc_vxc(x_func,1,rhop(1,ii),ex(ii),vxp(1,ii))
      call xc_f90_lda_exc_vxc(c_func,1,rhop(1,ii),ec(ii),vcp(1,ii))

      exc(ii)=ex(ii)+ec(ii)
      write(*,'(4f14.4)') rs(ii),  exc(ii),  vxp(1,ii)+vcp(1,ii),vxp(2,ii)+vcp(2,ii) 
   end do

   stop

!-----------------------------------------------------------------------
! LDA (Slater+PZ), intermediate-polarization case, numerical derivatives
!-----------------------------------------------------------------------

   SPIN=XC_POLARIZED
   call xc_f90_func_init(x_func, x_info, XC_LDA_X, SPIN)
   call xc_f90_func_init(c_func, c_info, XC_LDA_C_PZ, SPIN)

   !zeta = 1.000d0
   write(*,*)
   write(*,*) '# k.) Slater(X)+PZ(C) intermediate polarization:'
   write(*,*) '# numerical derivatives'
   write(*,*) '# zeta=',zeta
   write(*,*) '#         rs           exc           vxc1      vxc2'
   do ii=1,7
      rho(ii) = 3.0d0/( 4.0d0 * pi * rs(ii) * rs(ii) * rs(ii) )
      drho = 0.005d0*rho(ii)

      do kk=1,5
        rhop(1,ii) = rho(ii)*(1.d0+zeta)/2.d0 + drho*dble(kk-3)
        rhop(2,ii) = rho(ii)*(1.d0-zeta)/2.d0 
        call xc_f90_lda_exc_vxc(x_func,1,rhop(1,ii),ex(ii),vxp(1,ii))
        call xc_f90_lda_exc_vxc(c_func,1,rhop(1,ii),ec(ii),vcp(1,ii))
        exctmp(1,kk) = (ex(ii) + ec(ii))*(rho(ii) + drho*dble(kk-3))
      enddo

      do kk=1,5
        rhop(1,ii) = rho(ii)*(1.d0+zeta)/2.d0
        rhop(2,ii) = rho(ii)*(1.d0-zeta)/2.d0 + drho*dble(kk-3)
        call xc_f90_lda_exc_vxc(x_func,1,rhop(1,ii),ex(ii),vxp(1,ii))
        call xc_f90_lda_exc_vxc(c_func,1,rhop(1,ii),ec(ii),vcp(1,ii))
        exctmp(2,kk) = (ex(ii) + ec(ii))*(rho(ii) + drho*dble(kk-3))
      enddo

      exc(ii)=exctmp(1,3)/rho(ii)
      vxcp(1,ii) = exctmp(1,1)-8.d0*exctmp(1,2)+8.d0*exctmp(1,4)-exctmp(1,5)
      vxcp(1,ii) = vxcp(1,ii)/(12.d0*drho)
      vxcp(2,ii) = exctmp(2,1)-8.d0*exctmp(2,2)+8.d0*exctmp(2,4)-exctmp(2,5)
      vxcp(2,ii) = vxcp(2,ii)/(12.d0*drho)
      write(*,'(4f14.4)') rs(ii),  exc(ii), vxcp(1,ii), vxcp(2,ii)
   end do

   stop
!-----------------------------------------------------------------------  
! LDA (Slater+PZ), unpolarized case
!-----------------------------------------------------------------------

   SPIN=XC_UNPOLARIZED
   write(*,*)
   write(*,*) '# l.) Slater(X)+PZ(C), unpolarized:'
   write(*,*) '# analytical derivatives'
   write(*,*) '#         rs           exc           vxc'
  call xc_f90_func_init(x_func, x_info, XC_LDA_X, SPIN)
  call xc_f90_func_init(c_func, c_info, XC_LDA_C_PZ, SPIN)

   call xc_f90_lda_exc_vxc(x_func,7,rho(1),ex(1),vx(1))
   call xc_f90_lda_exc_vxc(c_func,7,rho(1),ec(1),vc(1))

   do ii = 1, 7
      exc(ii)=ex(ii)+ec(ii)
      vxc(ii)=vx(ii)+vc(ii)
      write(*,'(5f14.4)') rs(ii),  exc(ii),  vxc(ii)
   end do

!-----------------------------------------------------------------------  
! LDA (Slater+PZ), unpolarized case, numerical derivative
!-----------------------------------------------------------------------

   SPIN=XC_UNPOLARIZED
   write(*,*)
   write(*,*) '# m.) Slater(X)+PZ(C), unpolarized:'
   write(*,*) '# numerical derivatives'
   write(*,*) '#         rs           exc           vxc'
   
  call xc_f90_func_init(x_func, x_info, XC_LDA_X, SPIN)
  call xc_f90_func_init(c_func, c_info, XC_LDA_C_PZ, SPIN)


   do ii=1,7
     rho(ii) = 3.0d0/( 4.0d0 * pi * rs(ii) * rs(ii) * rs(ii) )
     drho = 0.005d0*rho(ii)
     do kk=1,5  
       call xc_f90_lda_exc_vxc(x_func,1,rho(ii)+ drho*dble(kk-3),ex(ii),vx(ii))
       call xc_f90_lda_exc_vxc(c_func,1,rho(ii)+ drho*dble(kk-3),ec(ii),vc(ii))
       exctmp0(kk) = (ex(ii) + ec(ii))*(rho(ii)+ drho*dble(kk-3))
     enddo
     exc(ii) = exctmp0(3)/rho(ii)
     vxc(ii) = exctmp0(1)-8.d0*exctmp0(2)+8.d0*exctmp0(4)-exctmp0(5)
     vxc(ii) = vxc(ii)/(12.d0*drho)
     write(*,'(5f14.4)') rs(ii),  exc(ii),  vxc(ii)
   end do

end program lxcksdttest
